@@ -323,14 +323,17 @@ sub register {
 	my $cb = sub {
 		my ($self,$cb_final,$expires,$endpoint,$ctx,$errno,$code,$packet,$leg,$from) = @_;
 		if ( $code && $code =~m{^2\d\d} ) {
-			my $exp = $packet->get_header( 'Expires' );
-			if ( ! defined $exp ) {
-				foreach my $c ( $packet->get_header( 'contact' ) ) {
-					my ($addr,$p) = sip_hdrval2parts( contact => $c );
-					defined( my $e = $p->{expires} ) || next;
-					$exp = $e if ! defined($exp) || $e < $exp;
-				}
+			# use expires info on contact
+			# if none given use global expires header
+			# see rfc3261 10.3.8,10.2.4
+			my $exp;
+			for my $c ( $packet->get_header( 'contact' ) ) {
+				my ($addr,$p) = sip_hdrval2parts( contact => $c );
+				defined( my $e = $p->{expires} ) or next;
+				sip_uri_eq($addr,$contact) or next; # not me
+				$exp = $e if ! defined($exp) || $e < $exp;
 			}
+			$exp = $packet->get_header( 'Expires' ) if ! defined $exp;
 			$$expires = $exp;
 			invoke_callback( $cb_final, 'OK', expires => $exp );
 
@@ -392,6 +395,7 @@ sub invite {
 #      Net::SIP::Simple::Call to handle the data
 #    cb_established: callback called after receiving ACK
 #    cb_cleanup: called on destroy of call object
+#    auth_whatever: will require authorization, see whatever in Net::SIP::Authorize
 #    for all other args see Net::SIP::Simple::Call....
 # Returns: NONE
 ###########################################################################
@@ -447,9 +451,51 @@ sub listen {
 		$cb->( $call,$endpoint,$ctx,undef,undef,$request,$leg,$from );
 	};
 
-	$self->{endpoint}->set_application([ $receive, $self,\%args ]);
+	$self->{endpoint}->set_application( [ $receive,$self,\%args] );
+
+	# in case listener should provide authorization put Authorizer in between
+	if ( my $auth = _make_auth_from_args($self,\%args) ) {
+		$self->create_chain([$auth,$self->{endpoint}]);
+	}
 }
 
+
+###########################################################################
+# create authorization if args say so
+# Args: ($self,$args)
+#   %$args:
+#     auth_user2pass: see user2pass in Net::SIP::Authorize 
+#     auth_user2a1:   see user2a1 in Net::SIP::Authorize 
+#     auth_realm:     see realm in Net::SIP::Authorize 
+#     auth_.... :     see Net::SIP::Authorize
+# Returns: authorizer if auth_* args given, removes auth_ args from hash
+##########################################################################
+sub _make_auth_from_args {
+	my ($self,$args) = @_;
+
+	my %auth = 
+		map { m{^auth_(.+)} ? ($1 => delete $args->{$_}):() } 
+		keys %$args;
+	my $i_am_proxy = delete $auth{i_am_proxy};
+
+	return %auth && $self->create_auth(%auth);
+}
+
+###########################################################################
+# setup authorization for use in chain
+# Args: ($self,%args)
+#   %args:  see Net::SIP::Authorize
+# Returns: authorizer object
+##########################################################################
+sub create_auth {
+	my ($self,%args) = @_;
+	return Net::SIP::Authorize->new(
+		dispatcher => $self->{dispatcher},
+		%args,
+	);
+}
+
+
 ###########################################################################
 # setup a simple registrar
 # Args: ($self,%args)
@@ -458,31 +504,52 @@ sub listen {
 #     min_expires: minimum expires time accepted, default 30
 #     domains|domain: domain or \@list of domains the registrar is responsable
 #       for. special domain '*' catches all
+#    auth_whatever: will require authorization, see whatever in Net::SIP::Authorize
 # Returns: $registrar
 ###########################################################################
 sub create_registrar {
 	my Net::SIP::Simple $self = shift;
+	my %args = @_;
+	my $auth = _make_auth_from_args($self,\%args);
+
 	my $registrar = Net::SIP::Registrar->new(
 		dispatcher => $self->{dispatcher},
-		@_
+		%args
 	);
-	$self->{dispatcher}->set_receiver( $registrar );
+	if ( $auth ) {
+		$registrar = $self->create_chain(
+			[$auth,$registrar], 
+			methods => ['REGISTER']
+		)
+	} else {
+		$self->{dispatcher}->set_receiver( $registrar );
+	}
 	return $registrar;
 }
 
 ###########################################################################
 # setup a stateless proxy
 # Args: ($self,%args)
-#   %args: see Net::SIP::StatelessProxy
+#   %args: see Net::SIP::StatelessProxy, for auth_whatever see whatever 
+#      in Net::SIP::Authorize
 # Returns: $proxy
 ###########################################################################
 sub create_stateless_proxy {
 	my Net::SIP::Simple $self = shift;
+	my %args = @_;
+
+	$args{auth_i_am_proxy} = 1;
+	my $auth = _make_auth_from_args($self,\%args);
+
 	my $proxy = Net::SIP::StatelessProxy->new(
 		dispatcher => $self->{dispatcher},
-		@_
+		%args
 	);
-	$self->{dispatcher}->set_receiver( $proxy );
+	if ( $auth ) {
+		$proxy = $self->create_chain([$auth,$proxy])
+	} else {
+		$self->{dispatcher}->set_receiver($proxy);
+	}
 	return $proxy;
 }
 
